;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!RAM_AirMeter = $7F8827

	;; Possible values for the below variables:
	;;   (fast) $00, $01, $03, $07, $0f, $1f, $3f, $7f, $ff (slow)
	!CountDownFrequency = $03
	!CountUpFrequency = $00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
;; INIT and MAIN routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	LDA #$FF
	STA !RAM_AirMeter
	RTL

	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR MainSub
	PLB
	RTL
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite Main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MainSub:
	LDA !RAM_AirMeter	; Play warning sounds
	CMP #$1C
	BEQ Sound
	CMP #$14
	BEQ Sound
	CMP #$0C
	BNE NoSound
Sound:	
	LDA #$2a
	STA $1DFC
NoSound:
	LDA !RAM_AirMeter
	INC
	ORA $1540,x
	BEQ NoGfx
	
	JSR SubGfx		; Draw the sprite
NoGfx:	

	LDA $9D			; Return if sprites are locked
	BNE Return

	LDA $75
	BEQ CountUp
CountDown:
	LDA $13
	AND #!CountDownFrequency
	BNE Return

	LDA !RAM_AirMeter
	DEC A
	BNE SetMeter
	JSL $00F606
	RTS
	
CountUp:
	LDA $13
	AND #!CountUpFrequency
	BNE Return
	
	LDA !RAM_AirMeter
	INC
	BEQ Return

	LDA #$48
	STA $1540,x
	
	LDA !RAM_AirMeter
	INC A
	;; 	LDA #$FF		; NOTE: Reinstate to make the meter instantly reset when not in water
SetMeter:	
	STA !RAM_AirMeter
	
Return:	
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Graphics Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteTileDispX:
		db $00,$08,$10,$18,$20,$27
SpriteTileDispY:
		db $20,$18,$10,$08,$00,$F9
SpriteTileDispYHigh:
		db $00,$00,$00,$00,$00,$FF
SpriteGfxProp:
		db $80,$00,$00,$00,$00,$00

Palettes:
		db $09,$05,$0B
	
SubGfx:	JSR GetPalette
	;; 	LDA $15F6,x		; NOTE: Reinstate to use the palette from the cfg file
	STA $02
	
	LDA !RAM_AirMeter	; Load address to be timed
	LSR
	LSR
	LSR
	STA $03
	
	LDY $15EA,x	     ; get offset to sprite OAM			   

	PHX
	LDX #$05
GfxLoopStart:
	LDA $7E			;change this to adjust x position
	CLC			;of the meter with the player
	ADC #$14		;
	STA $0300,Y		;

	PHY
	PHP
	TYA
	LSR
	LSR
	TAY
	PLP
	LDA #$00
	BCS Set1
	BIT $7F
	BEQ Not1
Set1:	INC
Not1:	STA $0460,y
	PLY
	
	LDA $80
	CLC
	ADC SpriteTileDispY,x
	XBA
	LDA $81
	ADC SpriteTileDispYHigh,x
	XBA
	REP #$20
	PHA
	CLC
	ADC.w #$0010
	CMP.w #$0100
	PLA
	SEP #$20
	BCS NoTile
	STA $0301,Y

	LDA $03
	JSR CalculateTile
	STA $0302,Y
	
	LDA $02
	ORA SpriteGfxProp,x
	ORA #$20
	STA $0303,Y
	
NoTile:	INY
	INY
	INY
	INY

	DEX
	BPL GfxLoopStart
	
	PLX
	RTS

GetPalette:
	LDY #$00
	LDA !RAM_AirMeter
	CMP #$55
	BCC LoadPalette
	INY
	CMP #$AA
	BCC LoadPalette
	INY
LoadPalette:
	LDA Palettes,y
	RTS	
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
CalculateTile:
	CPX #$00
	BEQ EndPiece
	CPX #$05
	BEQ EndPiece

	PHA
	TXA
	DEC A
	ASL
	ASL
	ASL
	STA $00
	PLA

	SEC
	SBC $00
	BMI EmptyTile
	
	CMP #$08
	BCS FullTile

	AND #$07
	CLC
	ADC #$47
	CMP #$4B
	BCC Return0
	CLC
	ADC #$0C
Return0:
	RTS

FullTile:
	LDA #$46
	RTS
EmptyTile:
	LDA #$47
	RTS
EndPiece:
	LDA #$56
	RTS